clear all
run main_log

% This script generates a movie with overlaid figures, for use with the
% swarm project.
% This script assumes that your camera is mounted above the workspace,
% pointed directly down.  It also assumes that your camera has no
%  distortions, and that the x and y axes are perfectly horizonal
% and vertical, respectively.
% 
% To use first extract the frames from your movie, and have them named
% frame0.jpeg, frame1.jpeg, frame2.jpeg, etc.  Put all the frames in a
% folder called "Frames" and put the folder in the working directory.
%
% You also must copy in "main_log.m" generated by the data logger.  
% 
% We must now align the plotting axes with the real world axes in the
% video, so that the position and SCALE will be correct when we overly the
% figures.  To do this, we will use nine equally spaced points arranged in 
% a rectangle centered at the origin.  The alignment parameters are saved, 
% so you only need to do this once if you don't move the camera.  You can 
% make a short clip with marked points on the testbed just for alignment.
%
% To perform the alignment, 
% 1.Make sure RES is set to the width and height of the resolution of the
%   frames, in pixels (e.g. [width height]).
% 2.Go to DOT_X and DOT_Y and set these to the spacing between the nine
%   dots in the real world.
% 3.Set TEST_OVERLAY_FITTING = 1 and GEN_MOV = 0
%   and run the script.  This will plot a square and nine '+' points in
%   on the screen.  Make sure the square is a true square, and not
%   distorted.
% 4.Adjust X_OFFSET and Y_OFFSET until the center '+' is lined up with the
%   center dot (the origin of the real world coordinate system).
% 5.Adjust SCALE until the other 8 dots are well matched up.  SCALE will
%   scale the axis, so increasing SCALE will increase the range, not the
%   unit distance.
% 6.The overlay axes should how match the real world axes in the video.
%   

% To view the video:
% 1.Set FPS to the frame rate of your video.
% 2.Set VIDEO_TIME to the length you want for the output video.  Your video
% will start at the first frame as time=0 and keep end at VIDEO_TIME.
% If you enter a length that is too long, the program should trigger an
% error.
% 3.Set the flags to choose which additional components (ellipses, comm.
% links, etc.) you wish to plot.
% 4.Set TEST_OVERLAY_FITTING = 0 and GEN_MOV = 0 and run the script.
%
% If you are satisfied with the video, set GEN_MOV = 1 and run the script.


%%
%%%%%%%  Modify these parameters
  %Set to 1 to check and see if your coordinate are matched up with real
  %world coords (only first frame is run), else set to 0.
  % When TEST_OVERLAY_FITTING is 1, image will display a square and mark the
  % calibration dot positions.

    TEST_OVERLAY_FITTING = 0;       
    SCALE=1.15;    % SCALE dimensions of the axis
    X_OFFSET=0;   %offset the axis
    Y_OFFSET=-39;
    
    RES = [640 480]; %resolution of frames

    VIDEO_TIME = 75; %desired video duration in seconds
    FPS = 30;

    GOAL = [100 300 160000 40000 40000]; %Initial goal stats
    COMMR = Inf; %default COMMR
    
%    Set flags for plotting additional components
    DRAW_COMM_LINKS = 1;
    DRAW_SWARM_ELLIPSE = 1;
    DRAW_GOAL_ELLIPSE = 1;
    DRAW_INDIVIDUAL_ESTIMATES = 1;
    DRAW_ROBOT_MARKER_DOT = 1;
    
    COMM_LINE_THICKNESS = 2; %line thickness of communication lins
    DOT_MARKER_SIZE = 30;    %size of dot marker.
    
    %Make movie or movie frames
    GEN_MOV = 0;  %Generate a movie
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%    
    %%%%%%%%%%%%   Calculate the dot coords that coorespond to floor
    %%%%%%%%%%%%   markings
    
    DOT_X=635;  %x spacing of calibration dots in real world
    DOT_Y=508;  %y spacing of calibration dots in real world
    
    XL=-DOT_X;  %x coord of left col of dots
    XR=DOT_X;   %x coord of right col of dots
    YB=-DOT_Y;  %y coord of bottom row of dots
    YT=DOT_Y;   %y coord of top row of dots

    %%%%%%%%%%%%%%
    if GEN_MOV == 1
        makemovie(GEN_MOV, 'open','scratch\swarm_movie');
        warning('A movie will be generated!  Press any key to continue or CTRL+C to cancel...')
        pause
    end
    
%%%%%%%%%%%%%%  SCALE and fit the axis to reflect real-world coordinates   
    aspect_ratio = RES(1)/RES(2);
    X_OFFSET=X_OFFSET*SCALE;
    Y_OFFSET=Y_OFFSET*SCALE;
    XLIM_R=[-1000 1000]*SCALE-X_OFFSET;
    YLIM_R=(XLIM_R+X_OFFSET)/aspect_ratio-Y_OFFSET;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

%%%%%%%%% Calculate number of frames and video time
    NUM_FRAMES = FPS*VIDEO_TIME;
    
% Find time when first 'wake' command received.  Robots start moving on
% video at this time.
start_cmd_time=0;
for i = 1:length(agent0Data.time)
    if agent0Data.data(1,i)==2
        start_cmd_time = agent0Data.time(i);
        break;
    end
end
if start_cmd_time==0
    warning('Start command time not found, press any key to continue');
    pause
end

% shift start time to 0
for i = 1:length(agent_list);
    agents(i).time = agents(i).time-start_cmd_time;
end

% Find earliest of final time stamps
end_times = [];
for i = 1:length(agent_list)
    if agent_list(i)~=0;
        end_times(end+1) = agents(i).time(end);
    end
end
first_end = min(end_times);

% Check to see if timestamps end before desired video length.
if VIDEO_TIME > first_end
    error('Not enough data to generate such a long video')
end

%Check to see if there are enough frames for the video length
filename1=sprintf('Frames/frame%i.jpeg',FPS*VIDEO_TIME);
if exist(filename1)==0
    error('Not enough images to generate such a long video')
end

% Time stamp for frames
video_frame_time=0:(1/FPS):VIDEO_TIME;

% Interpolate average esimator states at each time frame
for i = 1:length(agent_list)
    if agent_list(i)~= 0
        temp_data= interp1(agents(i).time, agents(i).data',video_frame_time);
        agents(i).video_data = temp_data';
    end
end

% calculate actual average stats at each time frame
swarm_stat_avg=zeros(5,length(video_frame_time));
num_agents=0;
for i = 1:length(agent_list)
    if agent_list(i)~= 0
        swarm_stat_avg(1:2,:) = swarm_stat_avg(1:2,:)+agents(i).video_data(11:12,:);
        swarm_stat_avg(3,:) = swarm_stat_avg(3,:)+agents(i).video_data(11,:).^2;
        swarm_stat_avg(4,:) = swarm_stat_avg(4,:)+agents(i).video_data(11,:).*agents(i).video_data(12,:);
        swarm_stat_avg(5,:) = swarm_stat_avg(5,:)+agents(i).video_data(12,:).^2;
        num_agents = num_agents+1;
    end
end
swarm_stat_avg = swarm_stat_avg/num_agents;


% Make a list of gogal-changing commands
command_list.data=[];
command_list.time=[];

if agent_list(1) == 0
    for i=1:length(agents(1).data(1,:))
        if (agents(1).data(1,i)~=0)
            command_list.data(:,end+1) = agents(1).data(:,i);
            command_list.time(end+1) = agents(1).time(i);            
        end
    end
else
    warning('Cannnot find data from vision system.')
    pause
end

%% Figure and axes positions and sizes
    figurePosition=[100 100 RES(1) RES(2)];
    axis1Position=[0 0 RES(1) RES(2)];
    axis2Position=[0 0 RES(1) RES(2)];


%% Initialize figure window
figureH=figure('Units','Pixels','Position',figurePosition);
%% Make scratch directory if none exists
if exist('scratch','dir')~=7
    mkdir('scratch')
end

% index for commands
command_i = 1;

if TEST_OVERLAY_FITTING == 1
    NUM_FRAMES =1;
end

%% Create the frames
for frame_i = 1:NUM_FRAMES
    
    %check for goal commands, if time of command is less than current time
    %index, update the goal
    if (command_i <= length(command_list.time))
        if command_list.time(command_i) < ((frame_i-1)*1/FPS)
            switch command_list.data(1,command_i)
                case 3
                    GOAL = command_list.data(2:6,command_i)';
                case 4
                    GOAL(1:2) = command_list.data(2:3,command_i)';
                case 8
                    COMMR = command_list.data(2,command_i)';
                otherwise
                    warning('command not recognized or not action mapped')
            end
            command_i=command_i+1;
        end
    end
       
    disp(frame_i);
    
%% display image
    filename1=sprintf('Frames/frame%i.jpeg',frame_i); %point to the correct directory
    frame1=imread(filename1);
    axis1H=axes('Units','Pixels','Position',axis1Position);
    image(frame1);
    axis off

    
    
%% Set up Axes
    axis2H=axes('units','pixels','position',axis2Position);

% %% Additional plotting other than ellipses
%     line([0 0],[-1500 YB],'LineWidth',175,'Color',[1 0.5 0]);
%     line([0 0],[1500 YT],'LineWidth',175,'Color',[1 0.5 0]);
%     hold on

%% display goal ellipse    
    %draw GOAL ellipse
    if DRAW_GOAL_ELLIPSE == 1
        hold on
        drawellipse(GOAL(1), GOAL(2), GOAL(3), GOAL(4), GOAL(5),'r');
    end
%% display swarm ellipse
    if DRAW_SWARM_ELLIPSE == 1
        hold on
        if ~isnan(swarm_stat_avg(1,frame_i))    %because of interpolation, some data points will be out of range and be NaN
            drawellipse(swarm_stat_avg(1,frame_i),...
                swarm_stat_avg(2,frame_i),...
                swarm_stat_avg(3,frame_i)-swarm_stat_avg(1,frame_i)^2,...% Transfrom from 2nd moment to central 2nd moment
                swarm_stat_avg(4,frame_i)-swarm_stat_avg(1,frame_i)*swarm_stat_avg(2,frame_i),...
                swarm_stat_avg(5,frame_i)-swarm_stat_avg(2,frame_i)^2,'g');
            hold on
        end
    end

%% display individual estimate ellipse   
    if DRAW_INDIVIDUAL_ESTIMATES==1
        hold on
        for i = 1:length(agent_list)
            if (agent_list(i)~=0) && (~isnan(agents(i).video_data(1,frame_i)))%because of interpolation, some data points will be out of range and be NaN
                color = choose_color(i);
                agent_ellipse_h= drawellipse_fast(agents(i).video_data(1,frame_i),...
                                                            agents(i).video_data(3,frame_i),...
                                                            agents(i).video_data(5,frame_i),... % Transfrom from 2nd moment to central 2nd moment
                                                            agents(i).video_data(7,frame_i),...
                                                            agents(i).video_data(9,frame_i),...
                                                            color);
            end
        end
    end
    
%% display swarm ellipse   
    if TEST_OVERLAY_FITTING ==1
        hold on
        scatter([XL 0 XR XL 0 XR XL 0 XR ],[YB YB YB 0 0 0 YT YT YT],'+');
        hold on
        line([-400 400 400 -400 -400],[-400 -400 400 400 -400],'color','r'); 
    end
    set(axis2H,...
        'Xlim',XLIM_R,...
        'Ylim',YLIM_R,...
        'PlotBoxAspectRatio',[1 1 1],...
        'DataAspectRatio',[1 1 1]);
    axis off
    
%% display communication links
    if DRAW_COMM_LINKS == 1
        hold on
        for i = 1:length(agent_list)
            for j = i:length(agent_list)
                if (agent_list(i)~=0)&&(agent_list(j)~=0)&&(agent_list(j)~=agent_list(i))
                    agentipos = agents(i).video_data(11:12,frame_i);
                    agentjpos = agents(j).video_data(11:12,frame_i);
                    if bot_dist(agentipos, agentjpos)<= COMMR
                        line([agentipos(1) agentjpos(1)],[agentipos(2) agentjpos(2)],'Color',[0 0 0],'linewidth',COMM_LINE_THICKNESS);
                    end
                end
            end
        end
    end
                
 
%% draw robot marker dots
    if DRAW_ROBOT_MARKER_DOT == 1
        hold on
        for i = 1:length(agent_list)
            if agent_list(i)~=0
                line(agents(i).video_data(11,frame_i),agents(i).video_data(12,frame_i),...
                    'MarkerEdgeColor',choose_color(i),...
                    'MarkerFaceColor',choose_color(i),...
                    'Marker','.',...
                    'MarkerSize',DOT_MARKER_SIZE );
            end
        end
    end
    
%%  Make video
    if GEN_MOV == 1
        makemovie(GEN_MOV, 'addrate','0');
        pause(0.25);
    end
    
%%     delete axes to free up memory (important!)
    if TEST_OVERLAY_FITTING ~= 1
        drawnow;
        delete(axis1H);
        delete(axis2H);
    end
     

end

if GEN_MOV == 1
    makemovie(GEN_MOV, 'close','');
end
disp('done')

